/*
 * BrainFlash, the AS3 BrainFuck interpreter
 * Author: Andy Li
 * See http://blog.onthewings.net/2009/10/08/brainflash-the-as3-brainfuck-interpreter
 */
package net.onthewings.misc
{
	import flash.utils.ByteArray;
	
	public class BrainFlash
	{
		public var showInput:Boolean = true;
		
		public var memory:ByteArray;
		
		public function get pointer():Number {
			return memory.position;
		}
		public function set pointer(value:Number):void {
			memory.position = value;
		}
		
		public function set input(value:String):void {
			_input = value.split("");
			for (var i:Number = 0 ; i < _input.length ; ++i){
				_input[i] = (_input[i] as String).charCodeAt();
			}
		}
		
		public function get output():String {
			return _output;
		}
		
		public function BrainFlash(memory:ByteArray = null)
		{
			if (memory){
				this.memory = memory;
			} else {
				this.memory = new ByteArray();
				this.memory.length = 30000;
			}
			this.memory.position = 0;
			_originalMemory = new ByteArray();
			_originalMemory.writeBytes(this.memory);
			this.memory.position = 0;
			_originalMemory.position = 0;
		}
		
		public function clearOutput():void {
			_output = "";
		}
		
		public function resetMemory():void {
			memory.position = 0;
			memory.writeBytes(_originalMemory);
			memory.position = 0;
			_originalMemory.position = 0;
		}
		
		public function runProgram(program:String, input:String = null):void {
			this.input = input;
			_program = program;
			_programPosition = 0;
			clearOutput();
			
			while (_programPosition < _program.length){
				runCommand(_program.charAt(_programPosition));
			}
		}
		
		private var _output:String;
		private var _input:Array;
		private var _program:String;
		private var _programPosition:Number;
		private var _originalMemory:ByteArray;
		
		private function moveToNextCommand():String {
			++_programPosition;
			
			if (_programPosition >= _program.length){
				return null;
			}
			
			return (_program.charAt(_programPosition));
		}
		
		private function moveToPrevCommand():String {
			--_programPosition;
			
			if (_programPosition < 0){
				return null;
			}
			
			return (_program.charAt(_programPosition));
		}
		
		private function runCommand(command:String):void {
			var val:Number;
			var c:String;
			var count:Number;
			switch (command) {
				case '>': //Increment the pointer.
					++pointer;
					moveToNextCommand();
					break;
				case '<': //Decrement the pointer.
					--pointer;
					moveToNextCommand();
					break;
				case '+': //Increment the byte at the pointer.
					val = Math.min(memory.readByte()+1,255);
					--pointer;
					memory.writeByte(val);
					--pointer;
					moveToNextCommand();
					break;
				case '-': //Decrement the byte at the pointer.
					val = Math.max(memory.readByte()-1,0);
					--pointer;
					memory.writeByte(val);
					--pointer;
					moveToNextCommand();
					break;
				case '.': //Output the byte at the pointer.
					_output += String.fromCharCode(memory.readByte());
					--pointer;
					moveToNextCommand();
					break;
				case ',': //Input a byte and store it in the byte at the pointer.
					val = _input.shift();
					if (showInput){
						_output += String.fromCharCode(val);
						if (_input[0] && _input[0] == 13){
							_output += '\n';
						}
					}
					memory.writeByte(val);
					--pointer;
					moveToNextCommand();
					break;
				case '[': //Jump forward past the matching ] if the byte at the pointer is zero.
					val = memory.readByte();
					--pointer;
					if (val == 0){
						count = 1;
						while (c = moveToNextCommand()){
							if (c == '[') ++count;
							if (c == ']') --count;
							if (count == 0){
								moveToNextCommand();
								break;
							}
						}
					} else {
						moveToNextCommand();
					}
					break;
				case ']': //Jump backward to the matching [ unless the byte at the pointer is zero.
					val = memory.readByte();
					--pointer;
					if (val > 0){
						count = 1;
						while (c = moveToPrevCommand()){
							if (c == '[') --count;
							if (c == ']') ++count;
							if (count == 0){
								break;
							}
						}
					} else {
						moveToNextCommand();
					}
					break;
				default:
					moveToNextCommand();
			}
		}
	}
}